%{ open Ast %}

%token SEMI LPAREN RPAREN LBRACE RBRACE COMMA
%token PLUS MINUS TIMES DIVIDE ASSIGN
%token EQ NEQ LT LEQ GT GEQ
%token RETURN IF ELSE FOR WHILE INT
%token STRING FLOAT
%token LTABLE RTABLE TABLESEP 
/*%token LBRACK RBRACK */
%token PIPE
%token DOTOP
%token ROWAPPEND
%token COLAPPEND
%token FUNCTION
%token TABLE 
%token <float> FLOAT_LITERAL
%token <string> STRING_LITERAL
%token <int> LITERAL
%token <string> ID
%token EOF

%nonassoc NOELSE NOASSIGN
%nonassoc ELSE
%right ASSIGN
/*%left LOGICAL_AND LOGICAL_OR*/
%left EQ NEQ
%left LT GT LEQ GEQ
%left PLUS MINUS
%left TIMES DIVIDE

%start program
%type <Ast.program> program

%%

program:
   /* nothing */ { [], [] }
 | program tdecl { ($2 :: fst $1), snd $1 }
 | program fdecl { fst $1, ($2 :: snd $1) }

/* start modified part */
tdecl:
   	primitive_decl ASSIGN literal SEMI	{ SingleVarLiteral($1, $3) }
  | INT ID SEMI													{ SingleVarLiteral(($2,Int), LitInt(0)) }
  | STRING ID SEMI											{ SingleVarLiteral(($2,String), LitString("")) }
  | FLOAT ID SEMI												{ SingleVarLiteral(($2,Float), LitFloat(0.0)) }
	| TABLE ID ASSIGN LTABLE typelist RTABLE SEMI { FullTableLiteral($2, Signature(Array.of_list(List.rev $5)), [||]) }
	| TABLE ID ASSIGN LTABLE typelist TABLESEP tableliteral RTABLE SEMI { FullTableLiteral( $2, Signature(Array.of_list(List.rev $5)), Array.of_list(List.rev $7)) }

primitive_decl:
	  INT ID													{ ($2,Int) }
	|	STRING ID												{ ($2,String) }
	| FLOAT ID												{ ($2,Float) }
	
literal:
	  FLOAT_LITERAL										{ LitFloat($1) }
	| STRING_LITERAL									{ LitString($1) }
	| LITERAL													{ LitInt($1) }

typelist:
	| primitive_decl									{ $1 :: [] }
	| typelist COMMA primitive_decl 	{ $3 :: $1 }
	
tableliteral:
		valuesequence												{LiteralRow( Array.of_list (List.rev $1)) :: []}
  | tableliteral TABLESEP valuesequence	{LiteralRow( Array.of_list (List.rev $3)) :: $1}

valuesequence:
		literal													{[$1]}
	| valuesequence COMMA literal			{$3 :: $1}

fdecl:
	 FUNCTION returntype ID LPAREN formals_opt RPAREN LBRACE stmt_list RBRACE		
     { { fname = $3;
     returnsig = $2;
	 formals = $5;
	 body = List.rev $8 } }

returntype:
		INT								{ SingleVarLiteral(("",Int), LitInt(0)) }
	| STRING						{ SingleVarLiteral(("",String), LitString("")) }
	| FLOAT							{ SingleVarLiteral(("",Float), LitFloat(0.0)) }
	| TABLE LTABLE typelist RTABLE		{ FullTableLiteral("", Signature(Array.of_list(List.rev $3)), [||]) }

formals_opt:
    /* nothing */ { [] }
  | typelist   { List.rev $1 }

/*vdecl_list:*/
    /* nothing */ /*   { [] }
  | vdecl_list tdecl { $2 :: $1 } */

stmt_list:
    /* nothing */  { [] }
  | stmt_list stmt { $2 :: $1 }

stmt:
    expr SEMI { Expr($1) }
  | RETURN expr SEMI { Return($2) }
  | LBRACE stmt_list RBRACE { Block(List.rev $2) }
  | IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, Block([])) }
  | IF LPAREN expr RPAREN stmt ELSE stmt    { If($3, $5, $7) }
  | FOR LPAREN expr_opt SEMI expr_opt SEMI expr_opt RPAREN stmt
     { For($3, $5, $7, $9) }
  | WHILE LPAREN expr RPAREN stmt { While($3, $5) }
  | ID PIPE ID LBRACE stmt_list RBRACE { Pipe($1, $3, $5) }
  | tdecl { LocalDecl($1) }
  | ID ROWAPPEND ID SEMI  { RowAppend($1, $3) }
  | ID COLAPPEND ID	SEMI	{ ColAppend($1, $3) }

expr_opt:
    /* nothing */ { Noexpr }
  | expr          { $1 }

expr:
    literal          { Literal($1) }
  | ID               { Id($1) }
  | expr PLUS   expr { Binop($1, Add,   $3) }
  | expr MINUS  expr { Binop($1, Sub,   $3) }
  | expr TIMES  expr { Binop($1, Mult,  $3) }
  | expr DIVIDE expr { Binop($1, Div,   $3) }
  | expr EQ     expr { Binop($1, Equal, $3) }
  | expr NEQ    expr { Binop($1, Neq,   $3) }
  | expr LT     expr { Binop($1, Less,  $3) }
  | expr LEQ    expr { Binop($1, Leq,   $3) }
  | expr GT     expr { Binop($1, Greater,  $3) }
  | expr GEQ    expr { Binop($1, Geq,   $3) }
/*  | expr LOGICAL_AND expr { Binop($1, And_, $3) }
  | expr LOGICAL_OR expr { Binop($1, Or_, $3) } */
  | ID ASSIGN expr   { Assign($1, $3) }
  | ID DOTOP ID ASSIGN expr { TAssign($1,$3,$5) }
  | ID LPAREN actuals_opt RPAREN { Call($1, $3) }
  | LPAREN expr RPAREN { $2 }
  | ID DOTOP ID				{ GetCol($1, $3) }
/*  | ID LBRACK expr RBRACK { Filter($1, $3) } */

actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }
