{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "/*"     { comment lexbuf }           (* Comments *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| ';'      { SEMI }
| ','      { COMMA }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '='      { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "if"     { IF }
| "else"   { ELSE }
| "for"    { FOR }
| "while"  { WHILE }
| "return" { RETURN }
| "int"    { INT }
| "|"			 { PIPE }
| "#{"		 { LTABLE }
| "}#"		 { RTABLE }
| "#"			 { TABLESEP }
| "::"		 { COLAPPEND }
(*| "&&"		 { LOGICAL_AND }
| "||"		 { LOGICAL_OR } *)
| "<<"		 { ROWAPPEND }
| "."			 { DOTOP }
(*| "["			 { LBRACK }
| "]"			 { RBRACK } *)
| "string" { STRING }
| "float"  { FLOAT }
| "table"  { TABLE }
| "function" { FUNCTION }
| ['0'-'9']+"."['0'-'9']+ as float_lxm { FLOAT_LITERAL(float_of_string float_lxm) }
| "\""['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_' ' ']*"\"" as string_lxm { STRING_LITERAL(string_lxm) }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }
